@@ -34,8 +34,7 @@ def deal_with_stdout():
             for line in iter(process.stdout.readline, b""):
                 output += line.decode("utf-8")
         except Exception as e:
-            print("[-] Exiting thread due to: {}\n".format(str(e)))
-            sys.stdout.flush()
+            print_flush("[-] Exiting thread due to: {}\n".format(str(e)))
             break
 
 
@@ -59,8 +58,7 @@ async def send_status(message="", routing_key=""):
         message_body = aio_pika.Message(message.encode())
         await exchange.publish(message_body, routing_key=routing_key)
     except Exception as e:
-        print("[-] Exception in send_status: {}".format(str(e)))
-        sys.stdout.flush()
+        print_flush("[-] Exception in send_status: {}".format(str(e)))
 
 
 async def callback(message: aio_pika.IncomingMessage):
@@ -260,6 +258,11 @@ async def sync_classes():
         )
 
 
+def print_flush(message: str):
+    print(message)
+    sys.stdout.flush()
+
+
 async def rabbit_c2_rpc_callback(
     exchange: aio_pika.Exchange, message: aio_pika.IncomingMessage
 ):
@@ -278,10 +281,9 @@ async def rabbit_c2_rpc_callback(
                 routing_key=message.reply_to,
             )
         except Exception as e:
-            print(
+            print_flush(
                 "Exception trying to send message back to container for rpc! " + str(e)
             )
-            sys.stdout.flush()
 
 
 async def connect_and_consume_rpc(main_config: dict):
@@ -305,18 +307,15 @@ async def connect_and_consume_rpc(main_config: dict):
                 )
                 result = await asyncio.wait_for(task, None)
             except Exception as e:
-                print("Exception in connect_and_consume_rpc .consume: {}".format(str(e)))
-                sys.stdout.flush()
+                print_flush("Exception in connect_and_consume_rpc .consume: {}".format(str(e)))
         except (ConnectionError, ConnectionRefusedError) as c:
-            print("Connection to rabbitmq failed, trying again...")
-            sys.stdout.flush()
+            print_flush("Connection to rabbitmq failed, trying again...")
         except Exception as e:
-            print("Exception in connect_and_consume_rpc connect: {}\n trying again...".format(str(e)))
-            sys.stdout.flush()
+            print_flush("Exception in connect_and_consume_rpc connect: {}\n trying again...".format(str(e)))
         await asyncio.sleep(5)
 
 
-async def mythic_service():
+async def mythic_service(debug: bool):
     global hostname
     global exchange
     global container_files_path
@@ -334,42 +333,55 @@ async def mythic_service():
     container_files_path = container_files_path / "c2_code"
     while True:
         try:
+            if debug:
+                print_flush("Connecting to rabbitmq from mythic_service")
             connection = await aio_pika.connect_robust(
                 host=main_config["host"],
                 login=main_config["username"],
                 password=main_config["password"],
                 virtualhost=main_config["virtual_host"],
             )
+            if debug:
+                print_flush("Successfully connected to rabbitmq from mythic_service")
             try:
                 channel = await connection.channel()
+                if debug:
+                    print_flush("Declaring exchange in mythic_service")
                 exchange = await channel.declare_exchange(
                     "mythic_traffic", aio_pika.ExchangeType.TOPIC
                 )
+                if debug:
+                    print_flush("Declaring queue in mythic_service")
                 queue = await channel.declare_queue("", exclusive=True)
                 await queue.bind(
                     exchange="mythic_traffic", routing_key="c2.modify.{}.#".format(hostname)
                 )
                 # just want to handle one message at a time so we can clean up and be ready
                 await channel.set_qos(prefetch_count=30)
+                if debug:
+                    print_flush("Creating task to consume in mythic_service")
                 task = queue.consume(callback)
                 await sync_classes()
                 print("Listening for c2.modify.{}.#".format(hostname))
+                if debug:
+                    print_flush("Creating task for connect_and_consume_rpc from mythic_service")
                 task4 = asyncio.ensure_future(connect_and_consume_rpc(main_config))
+                if debug:
+                    print_flush("Waiting for all tasks to finish in mythic_service")
                 result = await asyncio.wait_for(task, task4)
+                if debug:
+                    print_flush("All tasks finished in mythic_service, repeating loop")
             except Exception as e:
-                print("Exception in mythic_service: " + str(traceback.format_exc()))
-                sys.stdout.flush()
+                print_flush("Exception in mythic_service: " + str(traceback.format_exc()))
         except (ConnectionError, ConnectionRefusedError) as c:
-            print("Connection to rabbitmq failed, trying again...")
-            sys.stdout.flush()
+            print_flush("Connection to rabbitmq failed, trying again...")
         except Exception as e:
-            print("Exception in mythic_service: " + str(traceback.format_exc()))
-            sys.stdout.flush()
+            print_flush("Exception in mythic_service: " + str(traceback.format_exc()))
         await asyncio.sleep(5)
     
 
 
-async def heartbeat_loop():
+async def heartbeat_loop(debug: bool):
     config_file = open("rabbitmq_config.json", "rb")
     main_config = json.loads(config_file.read().decode("utf-8"))
     config_file.close()
@@ -379,43 +391,48 @@ async def heartbeat_loop():
         hostname = main_config["name"]
     while True:
         try:
+            if debug:
+                print_flush("Connecting to rabbitmq from heartbeat_loop")
             connection = await aio_pika.connect_robust(
                 host=main_config["host"],
                 login=main_config["username"],
                 password=main_config["password"],
                 virtualhost=main_config["virtual_host"],
             )
+            if debug:
+                print_flush("Successfully connected to rabbitmq from heartbeat_loop")
             channel = await connection.channel()
-            # declare our heartbeat exchange that everybody will publish to, but only the apfell server will are about
+            if debug:
+                print_flush("Declaring exchange in heartbeat_loop for channel")
             exchange = await channel.declare_exchange(
                 "mythic_traffic", aio_pika.ExchangeType.TOPIC
             )
         except (ConnectionError, ConnectionRefusedError) as c:
-            print("Connection to rabbitmq failed, trying again...")
-            sys.stdout.flush()
+            print_flush("Connection to rabbitmq failed, trying again...")
             await asyncio.sleep(5)
             continue
         except Exception as e:
-            print("Exception in heartbeat_loop: " + str(traceback.format_exc()))
-            sys.stdout.flush()
+            print_flush("Exception in heartbeat_loop: " + str(traceback.format_exc()))
             await asyncio.sleep(5)
             continue
         while True:
             try:
                 # routing key is ignored for fanout, it'll go to anybody that's listening, which will only be the server
+                if debug:
+                    print_flush("Sending heartbeat in heartbeat_loop")
                 await exchange.publish(
                     aio_pika.Message("heartbeat".encode()),
                     routing_key="c2.heartbeat.{}".format(hostname),
                 )
                 await asyncio.sleep(10)
             except Exception as e:
-                print(str(e))
+                print_flush("Exception in heartbeat_loop trying to send heartbeat: " + str(e))
                 # if we get an exception here, break out to the bigger loop and try to connect again
                 break
 
 # start our service
-def start_service_and_heartbeat():
+def start_service_and_heartbeat(debug=False):
     loop = asyncio.get_event_loop()
-    loop.create_task(mythic_service())
-    loop.create_task(heartbeat_loop())
+    loop.create_task(mythic_service(debug))
+    loop.create_task(heartbeat_loop(debug))
     loop.run_forever()
